@@ -6,28 +6,31 @@ Reads one or more FASTQ files and reports some basic statistics.  Example:
 $ ./fastq_simple_stats.py my.fastq
 
 File: my.fastq
-Total sequence entries: 277971801
-Total bases: 28353123702
-Avg sequence length: 102.0
-GC percentage: 44.5
+     Total sequence entries: 277971801
+     Total bases: 28353123702
+     Avg sequence length: 102.0
+     GC percentage: 44.5
+
+If you pass multiple files via *.fastq or something similar this will report stats
+on these as an aggregate unless you pass the --individual flag, which will cause
+reporting on each file individually.
 
 Contact: jorvis@gmail.com
 """
 
-
 import argparse
 import codecs
 import gzip
 import os
 import sys
 
-
 def main():
     parser = argparse.ArgumentParser( description='Provides simple quantitative statistics for a given FASTQ file')
 
     ## output file to be written
     parser.add_argument('input_files', metavar='N', type=str, nargs='+', help='Path to one or more input files, separated by spaces' )
     parser.add_argument('-o', '--output_file', type=str, required=False, help='Optional path to an output file to be created, else prints on STDOUT' )
+    parser.add_argument('--individual', action='store_true', help='Report stats on each file individually' )
     args = parser.parse_args()
 
     ## open the output file
@@ -41,16 +44,21 @@ def main():
     entry_count = 0
     total_bases = 0
     gc_count    = 0
-    
-    line_number = 0
 
     for input_file in args.input_files:
+        line_number = 0
+
         if input_file.endswith('.gz'):
             fh = gzip.open( input_file, 'rb')
             is_compressed = True
         else:
-            fh = open( input_file, 'rU' )
+            fh = open(input_file, 'r')
             is_compressed = False
+
+        if args.individual:
+            entry_count = 0
+            total_bases = 0
+            gc_count    = 0
         
         for line in fh:
             if is_compressed:
@@ -70,18 +78,20 @@ def main():
                 gc_count += line.count('G')
                 gc_count += line.count('C')
 
+        if args.individual:
+            report_stats(fout, input_file, entry_count, total_bases, gc_count)
 
+    if not args.individual:
+        report_stats(fout, 'multiple', entry_count, total_bases, gc_count)
+
+def report_stats(fout, input_file, entry_count, total_bases, gc_count):
     avg_entry_length = total_bases / entry_count
 
-    if len(args.input_files) > 1:
-        fout.write("File: multiple\n")
-    else:
-        fout.write("File: {0}\n".format(args.input_files[0]))
-
-    fout.write("Total sequence entries: {0}\n".format(entry_count))
-    fout.write("Total bases: {0}\n".format(total_bases))
-    fout.write("Avg sequence length: {0:.1f}\n".format(avg_entry_length))
-    fout.write("GC percentage: {0:.1f}\n".format( (gc_count / total_bases) * 100))
+    fout.write("File: {0}\n".format(input_file))
+    fout.write("\tTotal sequence entries: {0}\n".format(entry_count))
+    fout.write("\tTotal bases: {0}\n".format(total_bases))
+    fout.write("\tAvg sequence length: {0:.1f}\n".format(avg_entry_length))
+    fout.write("\tGC percentage: {0:.1f}\n".format( (gc_count / total_bases) * 100))
     
 
 if __name__ == '__main__':
