@@ -1,3 +1,6 @@
+
+import itertools
+import sys
 import uuid
 
 #from biocode import utils, gff, tbl
@@ -9,8 +12,7 @@ import biocode.tbl
 Warning: this module requires Python 3.2 or higher
 
 Yes, there's a lot of abstraction that could happen here still but I'm trying to allow for
-special consideration to be allowed for each feature type and for calls to read more like
-biology than CS.
+special consideration for each feature type and for calls to read more like biology than CS.
 
 Each of the 'things' is intended to be biologically relevant, and I've included the definitions
 for each from the Sequence Ontology.  Their class representations may not match this perfectly
@@ -24,8 +26,6 @@ A set of abstract classes are defined first, then the other classes, all within
 within each group.
 '''
 
-
-
 class LocatableThing:
     '''
     Base class for any Things that are locatable on other Things, such as genes on contigs.
@@ -417,6 +417,9 @@ class Location:
     For forward strand features, phase is counted from the start field. For reverse strand features,
     phase is counted from the end field.
 
+    The possible values for strand are 1, 0 and -1.  1 is for forward features, -1 for reverse, and
+    0 is for unstranded features.
+
     The phase is REQUIRED for all CDS features.
     '''
     def __init__( self, on=None, fmin=None, fmin_partial=False, fmax=None, fmax_partial=False, phase=None, strand=None ):
@@ -507,11 +510,12 @@ class Assembly( LocatableThing ):
     aligning two or more different regions."
     '''
 
-    def __init__( self, id=None, locations=None, length=None, residues=None, children=None ):
+    def __init__( self, id=None, locations=None, length=None, residues=None, is_circular=None, children=None ):
         super().__init__(locations)
         self.id = id
         self.length = length
         self.residues = residues
+        self.is_circular = is_circular
         self.children = children
 
         if length is None and self.residues is not None and len(self.residues) > 0:
@@ -652,6 +656,7 @@ class Gene( LocatableThing ):
         self.children = _initialize_type_list(self.children, 'mRNA')
         self.children = _initialize_type_list(self.children, 'rRNA')
         self.children = _initialize_type_list(self.children, 'tRNA')
+        self.children = _initialize_type_list(self.children, 'tmRNA')
 
     def __hash__(self):
         return hash(self.id)
@@ -666,6 +671,8 @@ class Gene( LocatableThing ):
             self.add_rRNA(rna)
         elif isinstance(rna, tRNA):
             self.add_tRNA(rna)
+        elif isinstance(rna, tmRNA):
+            self.add_tmRNA(rna)
         else:
             raise Exception("ERROR: add_RNA() method called for unrecognized type: {0}".format(rna.__class__.__name__))
 
@@ -681,6 +688,10 @@ class Gene( LocatableThing ):
         rna.parent = self
         self.children['tRNA'].append(rna)
 
+    def add_tmRNA(self, rna):
+        rna.parent = self
+        self.children['tmRNA'].append(rna)
+
     def get_residues(self):
         if len(self.locations) == 0:
             raise Exception("ERROR: gene.get_residues() requested but gene {0} isn't located on anything.".format(self.id))
@@ -708,6 +719,9 @@ class Gene( LocatableThing ):
     def mRNAs(self):
         return self.children['mRNA']
 
+    def ncRNAs(self):
+        return self.children['tRNA'] + self.children['rRNA'] + self.children['tmRNA']
+    
     def polypeptides(self):
         """
         Returns a list() of all Polypeptide objects for all mRNA children.
@@ -721,7 +735,7 @@ class Gene( LocatableThing ):
         
 
     def RNAs(self):
-        return self.children['mRNA'] + self.children['tRNA'] + self.children['rRNA']
+        return self.children['mRNA'] + self.children['tRNA'] + self.children['rRNA'] + self.children['tmRNA']
 
     def remove_mRNA(self, rna):
         if rna in self.children['mRNA']:
@@ -840,7 +854,7 @@ class Match( LocatableThing ):
     The sequence ontology provides a host of specific match terms, but I'm using this
     as a lightweight implementation of them all rather than explicitly creating
     dozens of subclasses.  Instead, create a Match object and simply set any more
-    specific term using Match.class
+    specific term using Match.subclass
 
     Future mod: Add Annotation object to store info on the match?
     '''
@@ -982,7 +996,16 @@ class PolypeptideSet( MoleculeSet ):
         for seq_id in seqs:
             polypeptide = Polypeptide(id=seq_id, residues=seqs[seq_id]['s'])
             self.add(polypeptide)
-        
+
+class Region( LocatableThing ):
+    '''
+    So definition (2020-08-31): "A sequence_feature with an extent greater than zero. A nucleotide 
+    region is composed of bases and a polypeptide region is composed of amino acids."
+    '''
+    def __init__( self, id=None, locations=None ):
+        super().__init__(locations)
+        self.id = id
+            
 class RNA( LocatableThing ):
     '''
     SO definition (2013-05-22): "An attribute describing a sequence consisting of nucleobases
@@ -996,9 +1019,14 @@ class RNA( LocatableThing ):
         self.children = children
 
         ## This should be an instance of FunctionalAnnotation from annotation.py
-        #   It's considered best practice to put the annotation on the Polypeptide feature, when appropriate.
+        #   It's considered best practice to put the annotation on the Polypeptide feature for
+        #   coding RNAs.
+        #   Here we expect things like product (rRNAs), anticodon (tRNAs), etc.
         self.annotation = annotation
 
+        if annotation is not None:
+            print("Warning: The RNA.annotation attribute is deprecated and will soon be removed.  Instead, use ncRNA.annotation", file=sys.stderr)
+
         ## initialize any types needed
         self.children = _initialize_type_list(self.children, 'exon')
         self.children = _initialize_type_list(self.children, 'CDS')
@@ -1230,6 +1258,77 @@ class mRNA( RNA ):
         super().__init__(id, locations, parent, locus_tag, children)
         self.residues = residues
 
+    def longest_orf(self, require_start=False, frame=None):
+        '''
+        Searches all six frames (unless a specific frame is passed) and returns an 
+        open-reading frame (ORF) instance.
+
+        If you want to require a start codon pass require_start=True and the ORF will be 
+        checked for one of AUG/ATG, GUG/GTG or UUG/TTG.  
+
+        If the longest ORF is on the reverse strand the residues within the ORF object
+        are reverse complemented.
+        '''
+        if self.residues is None:
+            raise Exception("mRNA.longest_orf() called but mRNA.residues was empty")
+        
+        stops = ["TAG", "TGA", "TAA"]
+        starts = ["ATG", "GTG", "TTG"]
+        
+        if frame is None:
+            frames = [1,2,3,-1,-2,-3]
+        elif frame not in [1,2,3,-1,-2,-3]:
+            raise Exception("Invalid value for frames.  mRNA.longest_orf() only considers one of [1,2,3,-1,-2,-3]")
+        else:
+            frames = [frame,]
+
+        longest_residues = ''
+        longest_frame = None
+
+        working_seq = self.residues.upper()
+        working_seq.replace('U', 'T')
+        working_seq_revcomped = False
+
+        for frame in frames:
+            real_frame = frame
+            
+            if frame < 0:
+                if not working_seq_revcomped:
+                    working_seq = biocode.utils.reverse_complement(working_seq)
+                    working_seq_revcomped = True
+                frame = 0 - frame
+
+            current_residues = ''
+                
+            for i in range(frame - 1, len(working_seq), 3):
+                codon = working_seq[i:i+3]
+                ## if the current codon is a stop, record this if it's the longest and restart
+                if codon in stops:
+                    if len(current_residues) > len(longest_residues):
+                        if require_start:
+                            if current_residues[0:3] in starts:
+                                longest_residues = current_residues
+                                longest_frame = real_frame
+                        else:
+                            longest_residues = current_residues
+                            longest_frame = real_frame
+
+                    current_residues = ''
+                else:
+                    current_residues += codon
+
+            # handle the last case
+            if len(current_residues) > len(longest_residues):
+                if require_start:
+                    if current_residues[0:3] in starts:
+                        longest_residues = current_residues
+                        longest_frame = real_frame
+                else:
+                    longest_residues = current_residues
+                    longest_frame = real_frame
+
+        return ORF(parent=self, frame=longest_frame, residues=longest_residues)
+
 class mRNASet( MoleculeSet ):
     '''
     This is a convenience class for when operations need to be performed on a set of mRNAs
@@ -1251,16 +1350,62 @@ class mRNASet( MoleculeSet ):
         for seq_id in seqs:
             mRNA = mRNA(id=seq_id, residues=seqs[seq_id]['s'])
             self.add(mRNA)
-        
-class rRNA( RNA ):
+
+class ncRNA( RNA ):
+    '''
+    SO definition (2019-04-17): "An RNA transcript that does not encode for a protein rather the RNA 
+    molecule is the gene product."
+
+    An ncRNA object has an annotation associated with it, since for protein-coding genes it is added 
+    on the polypeptide level.
+    '''
+    def __init__( self, id=None, locations=None, parent=None, locus_tag=None, children=None, annotation=None ):
+        super().__init__(id, locations, parent, locus_tag, children)
+
+        ## this should be an instance of FunctionalAnnotation from annotation.py
+        self.annotation = annotation
+
+class ORF( LocatableThing ):
+    '''
+    SO definition (2020-04-14): "The in-frame interval between the stop codons of a reading frame 
+    which when read as sequential triplets, has the potential of encoding a sequential string of 
+    amino acids. TER(NNN)nTER."
+
+    Note that this doesn't mean the sequence begins with a start codon.  It's simply devoid of 
+    stops.
+
+    The parent attribute should point to an mRNA instance.
+    '''
+    def __init__(self, id=None, locations=None, parent=None, frame=None, residues=None):
+        super().__init__(locations)
+        self.parent = parent
+        self.frame = frame
+        self.residues = residues
+            
+class tmRNA( ncRNA ):
+    '''
+    SO definition (2017-11-29): "A tmRNA liberates a mRNA from a stalled ribosome. To accomplish this part 
+    of the tmRNA is used as a reading frame that ends in a translation stop signal. The broken mRNA is 
+    replaced in the ribosome by the tmRNA and translation of the tmRNA leads to addition of a proteolysis 
+    tag to the incomplete protein enabling recognition by a protease. Recently a number of permuted tmRNAs 
+    genes have been found encoded in two parts. tmRNAs have been identified in eubacteria and some 
+    chloroplasts but are absent from archeal and Eukaryote nuclear genomes."
+    '''
+    def __init__( self, id=None, locations=None, parent=None, locus_tag=None, children=None ):
+        super().__init__(id, locations, parent, locus_tag, children)
+            
+class rRNA( ncRNA ):
     '''
     SO definition (2013-05-22): "RNA that comprises part of a ribosome, and that can provide both
     structural scaffolding and catalytic activity."
     '''
-    def __init__( self, id=None, locations=None, parent=None, locus_tag=None, children=None ):
+    def __init__( self, id=None, locations=None, parent=None, locus_tag=None, children=None, annotation=None ):
         super().__init__(id, locations, parent, locus_tag, children)
 
-class tRNA( RNA ):
+        ## this should be an instance of FunctionalAnnotation from annotation.py
+        self.annotation = annotation
+
+class tRNA( ncRNA ):
     '''
     SO definition (2013-05-22): "Transfer RNA (tRNA) molecules are approximately 80 nucleotides in length.
     Their secondary structure includes four short double-helical elements and three loops (D, anti-codon,
@@ -1271,9 +1416,9 @@ class tRNA( RNA ):
     in two manners: having more than one tRNA (with a specific anti-codon) for a particular amino acid;
     and 'wobble' base-pairing, i.e. permitting non-standard base-pairing at the 3rd anti-codon position."
     '''
-    def __init__( self, id=None, locations=None, parent=None, locus_tag=None, children=None ):
+    def __init__( self, id=None, locations=None, parent=None, locus_tag=None, children=None, anticodon=None ):
         super().__init__(id, locations, parent, locus_tag, children)
-
+        self.anticodon = anticodon
 
 class UTR( LocatableThing ):
     '''
@@ -1286,7 +1431,6 @@ class UTR( LocatableThing ):
         self.parent = parent
         self.length = length
 
-
 class FivePrimeUTR( UTR ):
     '''
     SO definition (2014-06-12): "A region at the 5' end of a mature transcript (preceding the initiation
@@ -1295,7 +1439,6 @@ class FivePrimeUTR( UTR ):
     def __init__( self, id=None, locations=None, parent=None, length=None ):
         super().__init__(id, locations, parent, length)
 
-
 class ThreePrimeUTR( UTR ):
     '''
     SO definition (2014-06-12): "A region at the 3' end of a mature transcript (following the stop codon)
